#!/usr/bin/ruby
#
#   metrics-statful.rb
#
# DESCRIPTION:
#   Sensu handler to ingest metrics into statful (https://statful.com/).
#
# PLATFORMS:
#   Linux
#
# DEPENDENCIES:
#   gem: sensu-plugin
#   gem: statful-client
#

require 'sensu-handler'
require 'statful-client'

require_relative '../lib/logger_mixin.rb'

class SensuMetricsStatful < Sensu::Handler
  def initialize
    super

    @client_config = settings['metrics-statful']['client']

    # Always flush on every metric since the sensu's handler isn't long lived
    @client_config['flush_size'] = 1

    @handler_config = settings['metrics-statful']['handler']

    if logging?
      # Log to stdout in order to be sent to sensu's log
      @client_config['logger'] = Logger.new(STDOUT)
    end

    @statful = StatfulClient.new(@client_config)
  end

  def filter; end

  def handle
    check = @event['check']

    check_metric = check['name']
    check_timestamp = check['timestamp']
    check_status = check['status']

    if status_unknown_or_custom?
      if ingest_status?
        ingest_status("status.#{check_metric}", check_status, check_timestamp)
      end
    else
      if ingest_status?
        ingest_status("status.#{check_metric}", check_status, check_timestamp)
      end

      begin
        require_relative "../lib/parsers/#{check_metric}.rb"
        check_value = Statful::Parser.parse(check['output'])
        ingest_output("output.#{check_metric}", check_value, check_timestamp)
      rescue LoadError
        # Do not spam the log with unsupported checks messages
        # puts "Check #{check_metric} not supported"
        exit
      end
    end
  end

  private

  def logging?
    @handler_config['logging'].nil? ? false : @handler_config['logging']
  end

  def status_unknown_or_custom?
    @event['check']['status'] > 2
  end

  def ingest_status?
    @handler_config['status'].nil? ? false : @handler_config['status']
  end

  def ingest_status(metric, status, timestamp)
    @statful.put(metric, status, { :timestamp => timestamp });
  end

  def ingest_output(metric, value, timestamp)
    @statful.put(metric, value, { :timestamp => timestamp });
  end
end
